/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it
 * is applied to this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * AgroSense. If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.task.field;

import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.Point;
import java.awt.Image;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Serializable;
import java.util.Arrays;
import java.util.List;
import nl.cloudfarming.client.field.model.ActivityField;
import nl.cloudfarming.client.field.model.TreatmentZone;
import nl.cloudfarming.client.geoviewer.Geographical;
import nl.cloudfarming.client.model.ItemIdType;
import nl.cloudfarming.client.task.ExportableTask;
import nl.cloudfarming.client.util.UOM;
import org.geotools.coverage.grid.GridCoverage2D;

/**
 *
 * @author Timon Veenstra <monezz@gmail.com>
 * @author Maciek Dulkiewicz
 */
public class FieldTask extends ExportableTask {

    public static final double MIN_WORK_WIDTH = 5;
    public static final String MIME_TYPE = "application/x-agrosense-field-task";
    public static final String EXT = "ftk";
    public static final String PROP_TREATMENTZONE_VALUE = "treatmentZoneValue";
    public static final String PROP_UOM = "uom";
    private double workWidth = MIN_WORK_WIDTH;
    private UOM uom;

    public FieldTask(ActivityField activityField) {
        super(activityField.getURI(), new Class[]{ActivityField.class}, ItemIdType.FTK);
        ActivityField copy = new ActivityField(activityField);
        getContext().addItem(copy);
        setName(activityField.getName());
        recalculateAverageTznValue();
        ic.add(new FieldTaskGeographical(copy));
        addListeners(copy);
    }

    private void addListeners(ActivityField activityField) {
        TreatmentZoneValueListener listener = new TreatmentZoneValueListener();
        for (TreatmentZone treatmentZone : activityField.getTreatmentZones()) {
            treatmentZone.addPropertyChangeListener(TreatmentZone.PROP_VALUE, listener);
        }
    }

    @Override
    public double getWorkWidth() {
        return workWidth;
    }

    public void setWorkWidth(double workWidth) {
        double oldWorkWidth = this.workWidth;
        this.workWidth = workWidth;
        setRegenerateGrid(true);
        getPropertyChangeSupport().firePropertyChange(ExportableTask.PROP_WORK_WIDTH, oldWorkWidth, this.workWidth);
    }

    public UOM getUom() {
        return uom;
    }

    public void setUom(UOM uom) {
        UOM old = this.uom;
        this.uom = uom;
        getPropertyChangeSupport().firePropertyChange(PROP_UOM, old, uom);
    }

    protected class FieldTaskGeographical extends ExportableTask.ExportableTaskGeographical {
        private final ActivityField activityField;

        public FieldTaskGeographical(ActivityField activityField) {
            this.activityField = activityField;
            addListener();
        }
        
        private void addListener(){
            FieldTask.this.addPropertyChangeListener(new PropertyChangeListener() {
                private final List<String> repaintProperties = Arrays.asList(new String[]{
                    ExportableTask.PROP_REGENERATE_GRID, ExportableTask.PROP_EXPORT_PREVIEW_GRID});
                
                @Override
                public void propertyChange(PropertyChangeEvent evt) {
                    // notify views:
                    if (repaintProperties.contains(evt.getPropertyName())) {
                        getPropertyChangeSupport().firePropertyChange(Geographical.PROP_RENDER_OBJECT, null, null);
                    }
                }
            });
        }

        @Override
        public Point getCentroid() {
            return activityField.getCentroid();
        }

        @Override
        public Geometry getBoundingBox() {
            return activityField.getBoundingBox();
        }

        @Override
        public String getTooltipText() {
            return activityField.getTooltipText();
        }

        @Override
        public Image getIcon() {
            return activityField.getIcon();
        }

        @Override
        public String getIconLabel() {
            return activityField.getIconLabel();
        }

        private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
            in.defaultReadObject();
            addListener();
        }
    }

    private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
        in.defaultReadObject();
        ActivityField activityField = getContext().lookup(ActivityField.class);
        if (activityField != null) {
            addListeners(activityField);
        }
    }

    private class TreatmentZoneValueListener implements PropertyChangeListener {

        @Override
        public void propertyChange(PropertyChangeEvent evt) {
            if (TreatmentZone.PROP_VALUE.equals(evt.getPropertyName())) {
                setRegenerateGrid(true);
                recalculateAverageTznValue();
                getPropertyChangeSupport().firePropertyChange(PROP_TREATMENTZONE_VALUE, evt.getOldValue(), evt.getNewValue());
            }
        }
    }
}
